"""Duck types miscellany."""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

from collections import Callable as _Callable

from . import _core as _ducktypes_core
from .. import _converters


def _type_isboollike(type):
    try:
        nonzero_method = getattr(type, '__nonzero__')
    except AttributeError:
        return False
    return isinstance(nonzero_method, _Callable)


boollike = _ducktypes_core.ducktypeclass_fromconverter('boollike',
                                                       _converters.bool,
                                                       type_issubclass=
                                                           _type_isboollike)



"""Converters."""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

from ._core import *
from ._misc import *
from ._num import *
from ._re import *
from ._string import *



"""Converters core."""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

from collections import Callable as _Callable
from functools import wraps as _wraps
import sys as _sys
import types as _types

from .._annotated import AnnotatedType as _AnnotatedType
from ..._misc import safe_name as _safe_name


def converter(totype, convertible_type_description=None,
              convertible_value_description=None):
    """Define a converter function.

    This decorator wraps a function that converts input values to values of
    some type.  The resulting function behaves exactly like the wrapped
    function unless the latter indicates an invalid input by raising a
    :exc:`TypeError` or :exc:`ValueError`.  On invalid inputs, the resulting
    function raises a descriptive exception that uses the given annotations.

    The *totype* may be a class or a descriptive name.  In either case, it
    is used by the converter's exceptions to describe the intended output.

    The *convertible_type_description* and *convertible_value_description*
    are used by the converter's exceptions to describe the allowed input
    types and values.  Their default values are the corresponding properties
    of the given *totype* if it is an
    :class:`~spruce.lang._datatypes._annotated.AnnotatedType` subclass.  If
    a *convertible_type_description* is defined but a
    *convertible_value_description* is not, then the latter takes the form
    of the former with a preceding indefinite article.

    :param totype:
        The type to which values are converted.
    :type totype: :obj:`str` or :obj:`type`

    :param convertible_type_description:
        A noun phrase (without any leading article) that describes the types
        of input values that can be converted.
    :type convertible_type_description: :obj:`str` or null

    :param convertible_value_description:
        A noun phrase (with a leading article, if appropriate) that
        describes the input values that can be converted.
    :type convertible_value_description: :obj:`str` or null

    """
    def decorate_func(func):
        return _wraps(func)(Converter(func,
                                      totype=totype,
                                      convertible_type_description=
                                          convertible_type_description,
                                      convertible_value_description=
                                          convertible_value_description))
    return decorate_func


class Converter(object):

    """A converter function.

    .. seealso:: :func:`converter`

    """

    def __init__(self, func, totype, convertible_type_description=None,
                 convertible_value_description=None):

        self._func = func

        if isinstance(totype, (_types.TypeType, _types.ClassType)):
            self._totype = totype
        else:
            self._totype = None

        custom_annotated_totype = False
        if any(arg is not None for arg in (convertible_type_description,
                                           convertible_value_description)):
            custom_annotated_totype = True
        else:
            for methodname in ('convertible_type_description',
                               'convertible_value_description', 'displayname',
                               'type_error', 'value_error'):
                try:
                    method = getattr(totype, methodname)
                except AttributeError:
                    custom_annotated_totype = True
                    break
                else:
                    if not isinstance(method, _Callable):
                        custom_annotated_totype = True
                        break

        if custom_annotated_totype:
            try:
                totype_displayname = totype.displayname()
            except (AttributeError, TypeError):
                try:
                    totype_displayname = totype.__name__
                except (AttributeError, TypeError):
                    totype_displayname = str(totype)

            try:
                totype_name = totype.__name__
            except AttributeError:
                totype_name = _safe_name(totype_displayname)

            annotated_totype_name = 'ConverterAnnotated_' + totype_name

            annotated_totype_bases = []
            if self.totype:
                annotated_totype_bases.append(self.totype)
            annotated_totype_bases.append(_AnnotatedType)
            annotated_totype_bases = tuple(annotated_totype_bases)

            @classmethod
            def displayname(cls):
                return totype_displayname

            convertible_type_description_ = convertible_type_description
            @classmethod
            def convertible_type_description(cls):
                return convertible_type_description_

            convertible_value_description_ = convertible_value_description
            @classmethod
            def convertible_value_description(cls):
                return convertible_value_description_

            annotated_totype_attrs = {'displayname': displayname}
            if convertible_type_description_ is not None:
                annotated_totype_attrs['convertible_type_description'] = \
                    convertible_type_description
            if convertible_value_description_ is not None:
                annotated_totype_attrs['convertible_value_description'] = \
                    convertible_value_description

            self._annotated_totype = \
                type(annotated_totype_name, annotated_totype_bases,
                     annotated_totype_attrs)

        else:
            self._annotated_totype = totype

    def __call__(self, value, *args, **kwargs):

        try:
            return self.func(value, *args, **kwargs)

        except (TypeError, ValueError) as exc:
            if isinstance(exc, TypeError):
                # TODO: (Python 2.7.2)
                #type_ = value.__class__
                type_ = type(value)
                message = str(exc)
                if not message or message == repr(type_):
                    message = None

                exc = self.type_error(type_, message)

            elif isinstance(exc, ValueError):
                message = str(exc)
                if not message or message == repr(value):
                    message = None

                exc = self.value_error(value, message)

            raise exc, None, _sys.exc_info()[2]

    @property
    def annotated_totype(self):
        return self._annotated_totype

    @property
    def func(self):
        return self._func

    @property
    def totype(self):
        return self._totype

    def type_error(self, type, message=None):
        return self.annotated_totype.conversion_type_error(type, message)

    def value_error(self, value, message=None):
        return self.annotated_totype.conversion_value_error(value, message)



"""Exceptions"""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

import exceptions as _py_exc


class Exception(_py_exc.Exception):
    pass


class Error(RuntimeError, Exception):
    pass


class TypeError(_py_exc.TypeError, Error):

    def __init__(self, type, expected_type_displayname,
                 expected_type_description=None, message=None, *args):
        self._expected_type_displayname = unicode(expected_type_displayname)
        self._expected_type_description = \
            unicode(expected_type_description) \
                if expected_type_description is not None else None
        self._message = unicode(message) if message is not None else None
        self._type = type
        super(TypeError, self).__init__(type, expected_type_displayname,
                                        expected_type_description, message,
                                        *args)

    def __str__(self):
        message = 'invalid {} type {!r}'.format(self.expected_type_displayname,
                                                self.type)
        if self.message:
            message += ': {}'.format(self.message)
        if self.expected_type_description:
            message += '; expecting {}'.format(self.expected_type_description)
        return message

    @property
    def expected_type_displayname(self):
        return self._expected_type_displayname

    @property
    def expected_type_description(self):
        return self._expected_type_description

    @property
    def message(self):
        return self._message

    @property
    def type(self):
        return self._type


class ConversionTypeError(TypeError, Error):
    def __str__(self):
        message = 'cannot convert type {!r} to {}'\
                   .format(self.type, self.expected_type_displayname)
        if self.message:
            message += ': {}'.format(self.message)
        if self.expected_type_description:
            message += '; expecting {}'.format(self.expected_type_description)
        return message


class ValueError(_py_exc.ValueError, Error):

    def __init__(self, value, expected_type_displayname,
                 expected_value_description=None, message=None, *args):
        self._expected_type_displayname = unicode(expected_type_displayname)
        self._expected_value_description = unicode(expected_value_description)
        self._expected_value_description = \
            unicode(expected_value_description) \
                if expected_value_description is not None else None
        self._message = unicode(message) if message is not None else None
        self._value = value
        super(ValueError, self).__init__(value, expected_type_displayname,
                                         expected_value_description, message,
                                         *args)

    def __str__(self):
        message = 'invalid {} {!r}'.format(self.expected_type_displayname,
                                           self.value)
        if self.message:
            message += ': {}'.format(self.message)
        if self.expected_value_description:
            message += '; expecting {}'.format(self.expected_value_description)
        return message

    @property
    def expected_type_displayname(self):
        return self._expected_type_displayname

    @property
    def expected_value_description(self):
        return self._expected_value_description

    @property
    def message(self):
        return self._message

    @property
    def value(self):
        return self._value


class ConversionValueError(ValueError, Error):
    def __str__(self):
        message = 'cannot convert {!r} to {}'\
                   .format(self.value, self.expected_type_displayname)
        if self.message:
            message += ': {}'.format(self.message)
        if self.expected_value_description:
            message += '; expecting {}'.format(self.expected_value_description)
        return message



"""Python language extensions miscellany."""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

import re as _re


def safe_name(name, default_str='_'):
    """Convert a name to a safe object name.

    The object name is "safe" in the sense that it consists only of
    characters that are allowed in object names in Python source code.
    Unallowed characters in the given *name* are replaced with
    *default_str*.

    :param str name:
        A name.

    :param str default_str:
        A string with which to replace unallowed characters.

    :rtype: str

    """
    safe_name_ = _re.sub(r'^[^\w]', default_str, name)
    safe_name_ = _re.sub(r'[^\d\w]', default_str, safe_name_)
    if not safe_name_:
        raise ValueError('cannot convert {!r} to a safe object name'
                             .format(name))
    return safe_name_


def safe_classname(name, default_str='_'):
    """Convert a name to a safe class name.

    This function passes its arguments through :func:`safe_name` and
    converts the result to CamelCase.

    :rtype: str

    """
    classname = ''.join(word.title() for word in safe_name(name).split('_')
                        if word)
    if not classname:
        raise ValueError('cannot convert {!r} to a safe class name'
                             .format(name))
    return classname



"""Data types."""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

from ._annotated import *
from ._checking import *
from ._converters import *
from ._ducktypes import *
from ._misc import *



"""Duck types core."""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

import __builtin__
import abc as _abc
from collections import Callable as _Callable

from .._annotated import AnnotatedType as _AnnotatedType


def ducktypeclass_fromconverter(classname, converter, bases=(),
                                type_issubclass=None):

    @classmethod
    def _value_isinstance(cls, value):
        try:
            converter(value)
        except (TypeError, ValueError):
            return False
        else:
            return True

    _type_issubclass = None
    if type_issubclass:
        @classmethod
        def _type_issubclass(cls, type):
            return type_issubclass(type)
    elif converter.totype:
        @classmethod
        def _type_issubclass(type):
            return issubclass(type, converter.totype)
    elif any(hasattr(base, '_type_issubclass') for base in bases):
        pass
    else:
        method = None
        for base in bases:
            try:
                method = getattr(base, '_type_issubclass')
            except AttributeError:
                pass
            else:
                if not isinstance(method, _Callable):
                    method = None

        if not method:
            if classname:
                ducktypeclass_description = 'duck type ' + classname
            else:
                ducktypeclass_description = 'a duck type'
            ducktypeclass_description += ' from converter {!r}'\
                                          .format(converter)
            if bases:
                ducktypeclass_description += ' with bases {}'.format(bases)

            raise ValueError('cannot determine a subclass check function for'
                              ' constructing {}'
                              .format(ducktypeclass_description))

    if not classname:
        classname = converter.annotated_totype.__name__ + '_DuckType'

    ducktypeclass_bases = [converter.annotated_totype]
    ducktypeclass_bases.extend(list(bases or (AnnotatedDuckType,)))
    ducktypeclass_bases = tuple(ducktypeclass_bases)

    ducktypeclass_attrs = {'_value_isinstance': _value_isinstance}
    if _type_issubclass:
        ducktypeclass_attrs['_type_issubclass'] = _type_issubclass

    return type(classname, ducktypeclass_bases, ducktypeclass_attrs)


def instance_of(type, displayname=None):

    if displayname is not None:
        def displayname_(cls):
            return displayname
    else:
        def displayname_(cls):
            return 'instance of {!r}'.format(type)
    displayname_.__name__ = 'displayname'
    displayname_ = classmethod(displayname_)

    @classmethod
    def _wrapped_type(cls):
        return type

    classname = type.__name__ + '_Instance'
    class_attrs = {'_wrapped_type': _wrapped_type}
    if displayname_ is not None:
        class_attrs['displayname'] = displayname_
    return __builtin__.type(classname, (InstanceOfType,), class_attrs)


def subclass_of(type, displayname=None):

    if displayname is not None:
        def displayname_(cls):
            return displayname
    else:
        def displayname_(cls):
            return 'subclass of {!r}'.format(type)
    displayname_.__name__ = 'displayname'
    displayname_ = classmethod(displayname_)

    @classmethod
    def _wrapped_type(cls):
        return type

    classname = type.__name__ + '_Subclass'
    class_attrs = {'_wrapped_type': _wrapped_type}
    if displayname_ is not None:
        class_attrs['displayname'] = displayname_
    return __builtin__.type(classname, (SubclassOfType,), class_attrs)


class DuckTypeMeta(_abc.ABCMeta):

    def __instancecheck__(self, value):
        return self._value_isinstance(value)

    def __subclasscheck__(self, type):
        return self._type_issubclass(type)


class AnnotatedDuckType(_AnnotatedType):

    __metaclass__ = DuckTypeMeta

    @classmethod
    @_abc.abstractmethod
    def _type_issubclass(cls, type):
        pass

    @classmethod
    @_abc.abstractmethod
    def _value_isinstance(cls, value):
        pass


class InstanceOfType(AnnotatedDuckType):

    __metaclass__ = DuckTypeMeta

    @classmethod
    def convertible_type_description(cls):
        return repr(cls._wrapped_type())

    @classmethod
    def _type_issubclass(cls, type):
        return issubclass(type, cls._wrapped_type())

    @classmethod
    def _value_isinstance(cls, value):
        return isinstance(value, cls._wrapped_type())

    @classmethod
    @_abc.abstractmethod
    def _wrapped_type(cls):
        pass


class SubclassOfType(AnnotatedDuckType):

    __metaclass__ = DuckTypeMeta

    @classmethod
    def convertible_type_description(cls):
        return 'a metaclass'

    @classmethod
    def convertible_value_description(cls):
        return 'a subclass of {!r}'.format(cls._wrapped_type())

    @classmethod
    def _type_issubclass(cls, type):
        return issubclass(type, __builtin__.type)

    @classmethod
    def _value_isinstance(cls, value):
        try:
            return issubclass(value, cls._wrapped_type())
        except TypeError:
            return False

    @classmethod
    @_abc.abstractmethod
    def _wrapped_type(cls):
        pass



"""Regular expression converters."""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

import re as _re

from . import _core as _converters_core


@_converters_core.converter('regular expression')
def regex(value):
    return _re.compile(value)



"""Higher-order functions."""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

from collections import namedtuple as _namedtuple
from functools import update_wrapper as _update_wrapper
from threading import Lock as _Lock


def lru_cache(maxsize=100, typed=False):

    """Memoize the wrapped function's most recent results.

    This caches the most recent *maxsize* results of the wrapped function.
    This can save time when an expensive or I/O bound function is
    periodically called with the same arguments.

    The arguments passed to the cached function must be hashable.

    If *maxsize* is null, then the LRU feature is disabled and the cache can
    grow without bound.  The LRU feature performs best when *maxsize* is a
    power of two.

    If *typed* is true, function arguments of different types will be cached
    separately.  For example, :code:`f(3)` and :code:`f(3.0)` will be
    treated as distinct calls with distinct results.

    To help measure the effectiveness of the cache and tune *maxsize*, the
    cached function is instrumented with a :meth:`!cache_info` method that
    returns a :func:`namedtuple <collections.namedtuple>` indicating
    :attr:`!hits`, :attr:`!misses`, :attr:`!maxsize`, and :attr:`!currsize`.
    In a multi-threaded environment, the :attr:`!hits` and :attr:`!misses`
    are approximate.

    The cached function also provides a :meth:`!cache_clear` method for
    clearing or invalidating the cache.

    The wrapped function is accessible through the cached function's
    :attr:`!__wrapped__` attribute.  This is useful for introspection, for
    bypassing the cache, or for rewrapping the function with a different
    cache.

    An `LRU (least recently used) cache`_ works best when the most recent
    calls are the best predictors of upcoming calls (for example, the most
    popular articles on a news server tend to change each day).  The cache's
    size limit assures that the cache does not grow without bound on
    long-running processes such as web servers.

    .. _LRU (least recently used) cache:
        http://en.wikipedia.org/wiki/Cache_algorithms#Least_Recently_Used

    .. note:: Source:
        :t:`Py2.6+ and Py3.0+ backport of Python 3.3's LRU Cache (Python
        recipe)`
        [`link
        <https://code.activestate.com/recipes/578078-py26-and-py30-backport-of-python-33s-lru-cache/>`_]

    """

    # Users should only access the lru_cache through its public API:
    #       cache_info, cache_clear, and f.__wrapped__
    # The internals of the lru_cache are encapsulated for thread safety and
    # to allow the implementation to change (including a possible C version).

    def decorating_function(user_function):

        cache = dict()
        # make statistics updateable non-locally
        stats = [0, 0]
        # names for the stats fields
        HITS, MISSES = 0, 1
        # separate positional and keyword args
        kwd_mark = (object(),)
        # bound method to lookup key or return None
        cache_get = cache.get
        # localize the global len() function
        _len = len
        # because linkedlist updates aren't threadsafe
        lock = _Lock()
        # root of the circular doubly linked list
        root = []
        # make updateable non-locally
        nonlocal_root = [root]
        # initialize by pointing to self
        root[:] = [root, root, None, None]
        # names for the link fields
        PREV, NEXT, KEY, RESULT = 0, 1, 2, 3

        def make_key(args, kwds, typed, tuple=tuple, sorted=sorted, type=type):
            # helper function to build a cache key from positional and keyword
            # args
            key = args
            if kwds:
                sorted_items = tuple(sorted(kwds.items()))
                key += kwd_mark + sorted_items
            if typed:
                key += tuple(type(v) for v in args)
                if kwds:
                    key += tuple(type(v) for k, v in sorted_items)
            return key

        if maxsize == 0:

            def wrapper(*args, **kwds):
                # no caching, just do a statistics update after a successful
                # call
                result = user_function(*args, **kwds)
                stats[MISSES] += 1
                return result

        elif maxsize is None:

            def wrapper(*args, **kwds):
                # simple caching without ordering or size limit
                key = make_key(args, kwds, typed) if kwds or typed else args
                # root used here as a unique not-found sentinel
                result = cache_get(key, root)
                if result is not root:
                    stats[HITS] += 1
                    return result
                result = user_function(*args, **kwds)
                cache[key] = result
                stats[MISSES] += 1
                return result

        else:

            def wrapper(*args, **kwds):
                # size limited caching that tracks accesses by recency
                key = make_key(args, kwds, typed) if kwds or typed else args
                with lock:
                    link = cache_get(key)
                    if link is not None:
                        # record recent use of the key by moving it to the
                        # front of the list
                        root, = nonlocal_root
                        link_prev, link_next, key, result = link
                        link_prev[NEXT] = link_next
                        link_next[PREV] = link_prev
                        last = root[PREV]
                        last[NEXT] = root[PREV] = link
                        link[PREV] = last
                        link[NEXT] = root
                        stats[HITS] += 1
                        return result
                result = user_function(*args, **kwds)
                with lock:
                    root = nonlocal_root[0]
                    if _len(cache) < maxsize:
                        # put result in a new link at the front of the list
                        last = root[PREV]
                        link = [last, root, key, result]
                        cache[key] = last[NEXT] = root[PREV] = link
                    else:
                        # use root to store the new key and result
                        root[KEY] = key
                        root[RESULT] = result
                        cache[key] = root
                        # empty the oldest link and make it the new root
                        root = nonlocal_root[0] = root[NEXT]
                        del cache[root[KEY]]
                        root[KEY] = None
                        root[RESULT] = None
                    stats[MISSES] += 1
                return result

        def cache_info():
            """Report cache statistics"""
            with lock:
                return _LruCacheInfo(stats[HITS], stats[MISSES], maxsize,
                                     len(cache))

        def cache_clear():
            """Clear the cache and cache statistics"""
            with lock:
                cache.clear()
                root = nonlocal_root[0]
                root[:] = [root, root, None, None]
                stats[:] = [0, 0]

        wrapper.__wrapped__ = user_function
        wrapper.cache_info = cache_info
        wrapper.cache_clear = cache_clear
        return _update_wrapper(wrapper, user_function)

    return decorating_function


_LruCacheInfo = \
    _namedtuple('CacheInfo', ['hits', 'misses', 'maxsize', 'currsize'])



"""Python language extensions.

This is a transitional project.  Each of these extensions will
eventually get migrated to its own project.

"""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__credits__ = ["Tyler Dumont", "Ivan D Vasin"]
__maintainer__ = "Ivan D Vasin"
__email__ = "nisavid@gmail.com"
__docformat__ = "restructuredtext"

from ._datatypes import *
from ._exc import *
from ._functools import *
from ._misc import *



"""Type checking."""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

from .. import _exc


def require_isinstance(value, type, type_displayname=None,
                       inst_description=None):
    if not isinstance(value, type):
        type_displayname = type_displayname or _type_displayname(type)

        if inst_description is None:
            try:
                inst_description = type.convertible_value_description()
            except (AttributeError, TypeError):
                pass

        raise _exc.ValueError(value, type_displayname, inst_description)


def require_issubclass(subtype, type, type_displayname=None,
                       type_description=None):
    if not issubclass(subtype, type):
        type_displayname = type_displayname or _type_displayname(type)

        if type_description is None:
            try:
                type_description = type.convertible_type_description()
            except (AttributeError, TypeError):
                pass

        raise _exc.TypeError(subtype, type_displayname, type_description)


def _type_displayname(type):
    if isinstance(type, tuple):
        type_displayname_items = []
        for type_item in type:
            try:
                type_item_displayname = type_item.displayname()
            except (AttributeError, TypeError):
                type_item_displayname = str(type_item)
            type_displayname_items.append(type_item_displayname)

        return str(tuple(type_displayname_items))

    else:
        try:
            return type.displayname()
        except (AttributeError, TypeError):
            try:
                return type.__name__
            except AttributeError:
                return str(type)



"""String converters."""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

from . import _core as _converters_core


@_converters_core.converter('hexadecimal integer')
def hex_int(value):
    return int(value, base=16)


@_converters_core.converter('string')
def string(value):
    return unicode(value)



"""Data types miscellany."""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

import abc as _abc
from collections import MutableSet as _MutableSet, Set as _Set
from functools import total_ordering as _total_ordering
import re as _re
from textwrap import dedent as _dedent

from . import _converters


class enum(object):

    """An enumerated type.

    An :class:`!enum` object is a named set of allowed values of any type.
    It provides a means to consistently identify such a set and validate
    that a given value exists in it.

    Calling an :class:`!enum` object on a value validates that the value
    exists in the enumeration.  The :attr:`enum_values` property provides
    access to the set of allowed values.

    For example, an :class:`!enum` over the set of card suits can be
    defined as follows::

        from spruce.lang import converter as _converter, enum as _enum

        @_converter('stripped lowercase string')
        def str_strip_lower(string):
            return string.strip().lower()

        CardSuit = _enum('card suit',
                         ('clubs', 'diamonds', 'hearts', 'spades'),
                         converter=str_strip_lower)

    The resulting class can be used to validate an ``input_suit`` like so::

        suit = CardSuit(input_suit)

    The resulting ``suit`` is guaranteed to be one of the values
    {``'clubs'``, ``'diamonds'``, ``'hearts'``, ``'spades'``}.  To iterate
    over the allowed suits, use ``CardSuit.enum_values``::

        print 'the {} is one of the following: {}'\\
               .format(CardSuit.enum_displayname,
                       ', '.join(CardSuit.displayname(value)
                                 for value in sorted(CardSuit.enum_values)))

    When using literal enum values, pass them through the enum class for
    early validation and self-documentation::

        cards = ((2, CardSuit('clubs')), (5, CardSuit('hearts')))

    :param str enum_displayname:
        A display name for this enum.

    :param values:
        The allowed values.
    :type values: ~[:obj:`object`]

    :param ord:
        The values' ordinals.
    :type ord: {:obj:`object`: :obj:`int`}

    :param converter:
        A converter from acceptable input values to the canonical *values*.
    :type converter: :obj:`object` -> :obj:`object`

    :param values_attrs:
        Other attributes of the values.
    :type values_attrs: {:obj:`object`: {:obj:`object`: :obj:`object`}}

    """

    def __init__(self, enum_displayname, values, ord=None, converter=None,
                 **values_attrs):

        self._enum_displayname = enum_displayname

        if not ord:
            ord = dict(zip(values, range(len(values))))
        self._ord = ord

        self._enum_values = sorted(values, key=(lambda value: ord[value]))

        for attrname, attrvalue_by_value in values_attrs.iteritems():
            priv_attrname = '_' + attrname
            setattr(self, priv_attrname, dict(attrvalue_by_value))

            def attr(value):
                return getattr(self, priv_attrname)[value]
            attr.__name__ = attrname

            setattr(self, attrname, attr)

        @_converters.converter(enum_displayname,
                               convertible_value_description=
                                   'one of the values {{{}}}'
                                    .format(', '.join(repr(value)
                                                      for value
                                                      in self.enum_values)))
        def converter_(value):

            if converter:
                canon_value = converter(value)
            else:
                canon_value = value

            if canon_value not in self.enum_values:
                exc = None
                if converter:
                    try:
                        converter_totype_displayname = \
                            converter.annotated_totype.displayname()
                    except (AttributeError, TypeError):
                        exc = ValueError('failed after converting {!r} to'
                                         ' {!r}'
                                          .format(value, canon_value))
                    else:
                        exc = ValueError\
                                  ('failed after converting {!r} to {} {!r}'
                                    .format(value,
                                            converter_totype_displayname,
                                            canon_value))

                if not exc:
                    exc = ValueError

                raise exc

            return canon_value
        self._converter_func_ = converter_

    def __call__(self, value):
        return self._converter_func(value)

    def __instancecheck__(self, value):
        try:
            self._converter_func(value)
        except ValueError:
            return False
        else:
            return True

    def __subclasscheck__(self, type):
        try:
            type_enum_values = type.enum_values
        except AttributeError:
            return False
        return set(type_enum_values) >= set(self.enum_values)

    def displayname(self, value):
        return self._displaynames[self(value)]

    @property
    def enum_displayname(self):
        return self._enum_displayname

    @property
    def enum_values(self):
        return self._enum_values

    def ord(self, value):
        return self._ord[self(value)]

    @property
    def _converter_func(self):
        return self._converter_func_


def namedflagset_classes(classname, frozenclassname=None, baseclassname=None,
                         doc=None, frozendoc=None):

    """Classes for sets of named flags.

    This method creates a class that defines a set of flags that are valid
    for its instances.  Objects of this class contain zero or more of these
    registered flags.  These objects support the full
    :class:`collections.MutableSet` API.

    """

    if baseclassname is None:
        baseclassname = classname + 'ABC'
    if frozenclassname is None:
        frozenclassname = 'Frozen' + classname

    __doc__suffix = \
        """

        .. seealso:: :func:`spruce.lang.namedflagset_classes
                            <spruce.lang._datatypes.namedflagset_classes>`

        :param flags:
            Flags.  A union (using ``|``) of zero or more of the flags
            registered via :meth:`register_flags`.
        :type flags: ~\ :obj:`int` or :obj:`None`

        :raise TypeError:
            Raised if non-null *flags* are given that are of a type that cannot
            be converted to an :obj:`int`.

        :raise ValueError:
            Raised if non-null *flags* are given that cannot be converted to an
            :obj:`int`.

        """
    __doc__suffix = _dedent(__doc__suffix)
    basedoc_shortdoc = \
        """A set of named flags.

        .. seealso:: :class:`{}`, :class:`{}`
        """\
         .format(classname, frozenclassname)
    baseclass__doc__ = _dedent(basedoc_shortdoc) + __doc__suffix

    __metaclass__ = _abc.ABCMeta

    def __init__(self, flags=None):
        if not flags:
            flags = 0
        try:
            flags = int(flags)
        except TypeError:
            raise TypeError('invalid flags type {!r}; expecting one that is'
                             ' accepted by {!r}'
                             .format(flags.__class__, int))
        except ValueError:
            raise ValueError('invalid flags {!r}; expecting an integer'
                              .format(flags))
        self._flags = flags

    def __and__(self, other):
        return self.__class__(int(self) & int(other))

    def __contains__(self, item):
        return item <= self

    def __eq__(self, other):
        return int(self) == int(other)

    def __int__(self):
        return self._flags

    def __iter__(self):
        for flag in self.valid_flags():
            if self.__class__(flag) in self:
                yield flag

    def __le__(self, other):
        return (int(self) & int(other)) == int(self)

    def __len__(self):
        return sum(1 for flag in self.valid_flags()
                   if self.__class__(flag) in self)

    def __nonzero__(self):
        return bool(int(self))

    def __or__(self, other):
        return self.__class__(int(self) | int(other))

    def __repr__(self):
        return '{}({})'.format(self.__class__.__name__,
                               ' | '.join(self.flag_name(flag)
                                          for flag in self._repr_flags))

    def __sub__(self, other):
        return self.__class__(int(self) - int(other))

    def __str__(self):
        return '{{{}}}'.format(', '.join(self.flag_displayname(flag)
                                         for flag in self._repr_flags))

    def __xor__(self, other):
        return self.__class__(int(self) ^ int(other))

    def copy(self):
        return self.__class__(self)

    @classmethod
    def flag_displayname(cls, flag):
        try:
            return cls._flags_displaynames[flag]
        except KeyError:
            raise ValueError('invalid {} flag {!r}'.format(cls.__name__, flag))

    @classmethod
    def flag_name(cls, flag):
        try:
            return cls._flags_names[flag]
        except KeyError:
            raise ValueError('invalid {} flag {!r}'.format(cls.__name__, flag))

    @classmethod
    def register_flag(cls, name, displayname=None, implied=None):

        flag = cls._frozenclass(cls._reserve_next_flag_value())
        if implied:
            flag |= implied

        cls._flags_names[flag] = name

        if displayname is None:
            displayname = name
        cls._flags_displaynames[flag] = displayname

        return flag

    @classmethod
    def valid_flags(cls):
        return cls._flags_names.keys()

    _flags_displaynames = {}

    _flags_names = {}

    _next_flag_value = 1

    @property
    def _repr_flags(self):
        flags = []
        f = int(self)
        for flag in reversed(sorted(int(flag) for flag in self.valid_flags())):
            if flag & f == flag:
                flags.append(flag)
                f -= flag
        return reversed(flags)

    @classmethod
    def _reserve_next_flag_value(cls):
        value = cls._next_flag_value
        cls._next_flag_value <<= 1
        return value

    baseclass_attrs = {'__doc__': baseclass__doc__,
                       '__metaclass__': __metaclass__,
                       '__init__': __init__,
                       '__and__': __and__,
                       '__contains__': __contains__,
                       '__eq__': __eq__,
                       '__int__': __int__,
                       '__iter__': __iter__,
                       '__le__': __le__,
                       '__len__': __len__,
                       '__nonzero__': __nonzero__,
                       '__or__': __or__,
                       '__repr__': __repr__,
                       '__str__': __str__,
                       '__xor__': __xor__,
                       'copy': copy,
                       'flag_displayname': flag_displayname,
                       'flag_name': flag_name,
                       'register_flag': register_flag,
                       'valid_flags': valid_flags,
                       '_flags_displaynames': _flags_displaynames,
                       '_flags_names': _flags_names,
                       '_next_flag_value': _next_flag_value,
                       '_repr_flags': _repr_flags,
                       '_reserve_next_flag_value': _reserve_next_flag_value,
                       }

    baseclass = _total_ordering(type(baseclassname, (_Set,), baseclass_attrs))

    if frozendoc is None:
        frozendoc = basedoc_shortdoc
    frozenclass__doc__ = _dedent(frozendoc) + __doc__suffix

    def __hash__(self):
        return hash(self._flags)

    frozenclass_attrs = {'__doc__': frozenclass__doc__, '__hash__': __hash__}
    frozenclass = type(frozenclassname, (baseclass,), frozenclass_attrs)
    baseclass._frozenclass = frozenclass

    if doc is None:
        doc = basedoc_shortdoc
    mutableclass__doc__ = _dedent(doc) + __doc__suffix

    def __iand__(self, other):
        self._flags &= int(other)
        return self

    def __ior__(self, other):
        self._flags |= int(other)
        return self

    def __isub__(self, other):
        self._flags -= int(other)
        return self

    def __ixor__(self, other):
        self._flags ^= int(other)
        return self

    def add(self, item):
        self._flags |= int(item)

    def discard(self, item):
        self._flags &= ~int(item)

    mutableclass_attrs = {'__doc__': mutableclass__doc__, '__iand__': __iand__,
                          '__ior__': __ior__, '__ixor__': __ixor__, 'add': add,
                          'discard': discard}
    mutableclass = type(classname, (baseclass, _MutableSet),
                        mutableclass_attrs)
    baseclass._mutableclass = mutableclass

    return baseclass, mutableclass, frozenclass


# TODO: (Python 2.7.2)
#regex_class = _re.compile('').__class__
regex_class = type(_re.compile(''))



"""Numeric duck types."""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

from collections import Callable as _Callable
import types as _types

from . import _core as _ducktypes_core
from .. import _converters


# float-like variants ---------------------------------------------------------


def _type_isfloatlike(type):
    return issubclass(type,
                      tuple([complex, float, int, long]
                            + list(_types.StringTypes)))


floatlike = _ducktypes_core.ducktypeclass_fromconverter('floatlike',
                                                        _converters.float,
                                                        type_issubclass=
                                                            _type_isfloatlike)


neg_floatlike = \
    _ducktypes_core.ducktypeclass_fromconverter('neg_floatlike',
                                                _converters.neg_float,
                                                bases=(floatlike,))


nonneg_floatlike = \
    _ducktypes_core.ducktypeclass_fromconverter('nonneg_floatlike',
                                                _converters.nonneg_float,
                                                bases=(floatlike,))


nonpos_floatlike = \
    _ducktypes_core.ducktypeclass_fromconverter('nonpos_floatlike',
                                                _converters.nonpos_float,
                                                bases=(floatlike,))


pos_floatlike = \
    _ducktypes_core.ducktypeclass_fromconverter('pos_floatlike',
                                                _converters.pos_float,
                                                bases=(floatlike,))


# int-like variants -----------------------------------------------------------


def _type_isintlike(type):
    try:
        intmethod = getattr(type, '__int__')
    except AttributeError:
        return issubclass(type, _types.StringTypes)
    else:
        return isinstance(intmethod, _Callable)


intlike = _ducktypes_core.ducktypeclass_fromconverter('intlike',
                                                      _converters.int,
                                                      type_issubclass=
                                                          _type_isintlike)


neg_intlike = _ducktypes_core.ducktypeclass_fromconverter('neg_intlike',
                                                          _converters.neg_int,
                                                          bases=(intlike,))


nonneg_intlike = \
    _ducktypes_core.ducktypeclass_fromconverter('nonneg_intlike',
                                                _converters.nonneg_int,
                                                bases=(intlike,))


nonpos_intlike = \
    _ducktypes_core.ducktypeclass_fromconverter('nonpos_intlike',
                                                _converters.nonpos_int,
                                                bases=(intlike,))


pos_intlike = _ducktypes_core.ducktypeclass_fromconverter('pos_intlike',
                                                          _converters.pos_int,
                                                          bases=(intlike,))



"""Annotated types."""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

import abc as _abc

from .. import _exc


class AnnotatedType(object):

    __metaclass__ = _abc.ABCMeta

    @classmethod
    def convertible_type_description(cls):
        return None

    @classmethod
    def convertible_value_description(cls):
        if cls.convertible_type_description():
            if any(cls.convertible_type_description().startswith(vowel)
                   for vowel in ('a', 'e', 'i', 'o', 'u')):
                article = 'an'
            else:
                article = 'a'

            return '{} {}'.format(article, cls.convertible_type_description())
        else:
            return None

    @classmethod
    def conversion_type_error(cls, type, message=None):
        return _exc.ConversionTypeError(type, cls.displayname(),
                                        cls.convertible_type_description(),
                                        message)

    @classmethod
    def conversion_value_error(cls, value, message=None):
        return _exc.ConversionValueError(value, cls.displayname(),
                                         cls.convertible_value_description(),
                                         message)

    @classmethod
    @_abc.abstractmethod
    def displayname(cls):
        pass

    @classmethod
    def type_error(cls, type, message=None):
        return _exc.TypeError(type, cls.displayname(),
                              cls.convertible_type_description(), message)

    @classmethod
    def value_error(cls, value, message=None):
        return _exc.ValueError(value, cls.displayname(),
                               cls.convertible_value_description(), message)



"""Numeric converters."""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

import __builtin__

from . import _core as _converters_core


# float variants --------------------------------------------------------------


@_converters_core.converter('floating point number')
def float(value):
    return __builtin__.float(value)


@_converters_core.converter('negative floating point number')
def neg_float(value):
    value = __builtin__.float(value)
    if value >= 0:
        raise ValueError
    return value


@_converters_core.converter('non-negative floating point number')
def nonneg_float(value):
    value = __builtin__.float(value)
    if value < 0:
        raise ValueError
    return value


@_converters_core.converter('non-positive floating point number')
def nonpos_float(value):
    value = __builtin__.float(value)
    if value > 0:
        raise ValueError
    return value


@_converters_core.converter('positive floating point number')
def pos_float(value):
    value = __builtin__.float(value)
    if value <= 0:
        raise ValueError
    return value


# int variants ----------------------------------------------------------------


@_converters_core.converter('integer')
def int(value):
    return __builtin__.int(value)


@_converters_core.converter('negative integer')
def neg_int(value):
    value = __builtin__.int(value)
    if value >= 0:
        raise ValueError
    return value


@_converters_core.converter('non-negative integer')
def nonneg_int(value):
    value = __builtin__.int(value)
    if value < 0:
        raise ValueError
    return value


@_converters_core.converter('non-positive integer')
def nonpos_int(value):
    value = __builtin__.int(value)
    if value > 0:
        raise ValueError
    return value


@_converters_core.converter('positive integer')
def pos_int(value):
    value = __builtin__.int(value)
    if value <= 0:
        raise ValueError
    return value



"""String-like duck types."""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

from collections import Callable as _Callable
import types as _types

from . import _core as _ducktypes_core
from .. import _converters


def _type_isstringlike(type):
    try:
        string_method = getattr(type, '__unicode__')
    except AttributeError:
        try:
            string_method = getattr(type, '__str__')
        except AttributeError:
            return issubclass(type, _types.StringTypes)
    return isinstance(string_method, _Callable)


hex_intlike = \
    _ducktypes_core.ducktypeclass_fromconverter('hexlike', _converters.hex_int,
                                                type_issubclass=
                                                    _type_isstringlike)


stringlike = \
    _ducktypes_core.ducktypeclass_fromconverter('stringlike',
                                                _converters.string,
                                                type_issubclass=
                                                    _type_isstringlike)



"""Regular-expression-like duck types."""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

from . import _core as _ducktypes_core
from . import _string as _ducktypes_string
from .. import _converters
from .. import _misc as _datatypes_misc


def _type_isregexlike(type):
    return issubclass(type, _datatypes_misc.regex_class) \
           or _ducktypes_string._type_isstringlike(type)


regexlike = _ducktypes_core.ducktypeclass_fromconverter('regexlike',
                                                        _converters.regex,
                                                        type_issubclass=
                                                            _type_isregexlike)



"""Converters miscellany."""

__copyright__ = "Copyright (C) 2014 Ivan D Vasin"
__docformat__ = "restructuredtext"

import __builtin__

from . import _core as _converters_core


@_converters_core.converter('boolean value')
def bool(value):
    return __builtin__.bool(value)
